var DDMailboxApp = DDMailboxApp || {};

DDMailboxApp.user = function(userId, sockId) {
    var self =this;

    self.userId = userId;
    self.sockId = sockId;
}

DDMailboxApp.chatMsg = function (txt, inbound) {
    var self = this;

    self.inbound = inbound;
    self.txt = txt;
}

DDMailboxApp.chatter = function () {
    var self = this;

    self.sockId = null;
    self.ws = null;

    self.messages = ko.observableArray();
    self.chatMessage = ko.observable('');
    
    self.handleChat = function () {
        self.ws.send(ko.toJSON({ MsgType: "SEND", ToSocket: self.sockId , Message: self.chatMessage }));
        self.messages.push(new DDMailboxApp.chatMsg(self.chatMessage(), false));
        self.chatMessage('');
    }

    self.setSockId = function (sockId) {
        self.sockId = sockId;
    }

    self.setWebsocket = function (ws) {
        self.ws = ws;
    }

    self.handleMessage = function(msg)
    {
        self.messages.push(new DDMailboxApp.chatMsg(msg, true));
    }

}
DDMailboxApp.VM = function () {
    var self = this;

    self.inited = false;
    self.url = null;
    self.ws = null;
    self.sockId = null;

    self.connected = ko.observable(false);
    self.sendBuffer = ko.observable("");

    self.chatter = new DDMailboxApp.chatter();
    self.chatting = ko.observable(false);

    self.users = ko.observableArray();
    self.otherUsers = ko.computed(function () {
        return ko.utils.arrayFilter(self.users(), function (item) { return item.sockId != self.sockId; });
    });

    //event handlers
    self.connect = function () {
        if(self.inited && self.ws.supported())
        {
            self.ws.connect();
            self.log('connection attempted');
        }
    }

    self.send = function (m) {
        self.ws.send(m);
        self.log('sent: ' + m);
    }

    self.close = function () {
        self.ws.close();
        self.log('close attempted');
    }
    self.log = function (msg) {
        if(console && console.log)
        {
            console.log(msg);
        }
    }

    self.visible = ko.computed(function () {
        return self.users().length > 1;
    });

    //event handlers
    self.handleOpen = function () {
        self.ws.send("MAILBOXOPENED");
    }
    self.handleClose = function () {
        self.ws.send("MAILBOXCLOSED");
    }
    self.handleUnload = function () {
        self.close();
    }
    self.handleUserClick = function (u) {
        self.chatter.setSockId(u.sockId)
        self.chatting(!self.chatting());
    }

    self.handleMessage = function (data) {
        try
        {
            if (data.substring(0, 1) == "{" || data.substring(0, 1) == "[") {
                var msg = JSON.parse(data);
                if (msg != null) {
                    switch (msg.MsgType) {
                        case "UPDATE":
                            //alert('count ' + self.users().length + ' ' + msg.Users.length);
                            self.users.removeAll();
                            if (msg.Users != null) {
                                $.each(msg.Users, function (index, val) {
                                    self.users.push(new DDMailboxApp.user(val.UserID, val.SockID));
                                });
                            }
                            break;
                        case "OPEN":
                            self.sockId = msg.SockID;
                            break;
                        case "CHAT":
                            self.chatter.handleMessage(msg.Message);
                            self.chatter.setSockId(msg.SockID);
                            self.chatting(true);
                            break;
                    }
                }
            }
        }
        catch(e)
        {
            alert('ex ' + e);
        }
    }

    //init
    self.init = function (url) {
        if (!self.inited) {
            self.inited = true;
            self.url = url;
            self.ws = new DDSocketApp.SocketBase(url, self.socketCallback);
            if (!self.ws.supported()) {
                alert('sockets not supported in this browser!');
            }
            else
            {
                self.chatter.setWebsocket(self.ws);
                //event binding
                $(document).bind("MAILBOXFORMOPENED", self.handleOpen)
                .bind("MAILBOXFORMCLOSED", self.handleClose)
                .unload(self.handleUnload);
            }
            //ko.cleanNode();
            ko.applyBindings(self, document.getElementById('concurrencyContainer') );
        }        
    }

    //callback
    self.socketCallback = function (socketEvent, event) {
        //DDSocketApp.OpenEvent = "opened";
        //DDSocketApp.CloseEvent = "close";
        //DDSocketApp.MessageEvent = "message";
        //DDSocketApp.ErrorEvent = "error";

        if (socketEvent == DDSocketApp.MessageEvent && event.data != null) {
            self.handleMessage(event.data);
            self.log('received: ' + event.data);
        }

        switch (socketEvent) {
            case DDSocketApp.OpenEvent:
                self.log('opened');
                self.connected(true);
                break;
            case DDSocketApp.CloseEvent:
            case DDSocketApp.ErrorEvent:
                self.connected(false);
                self.log('closed');
                break;
        }

    }
    self.notEmpty = function (txt) {
        return txt != null && txt != "";
    }
}

ko.bindingHandlers.slide = {
    init: function (element, valueAccessor, allBindings, viewModel, bindingContext) {

    },
    update: function (element, valueAccessor, allBindings, viewModel, bindingContext) {
        var value = ko.unwrap(valueAccessor());
        if (value) {
            $(element).slideDown();
        }
        else
        {
            $(element).slideUp();
        }
    }
};
