

var DDTestApp = DDTestApp || {};

DDTestApp.Msg = function(type,msg) {
    var self = this;

    self.date = new Date();
    self.type = type;
    self.msg = msg;

    if (self.type == null)
    {
        self.type = "unknown";
    }

    self.append = function (txt) {
        if (self.msg != null)
            self.msg += txt;
        else
            self.msg = txt;
    }
}

DDTestApp.VM = function () {
    var self = this;

    self.inited = false;
    self.url = null;
    self.ws = null;

    self.imageData = ko.observable();
    self.connected = ko.observable(false);
    self.messages = ko.observableArray();
    self.sendBuffer = ko.observable("");

    //event handlers
    self.execConnect = function () {
        self.ws.connect();
    }
    self.execClose = function () {
        self.ws.close();
    }
    self.execSend = function () {
        self.ws.send(self.sendBuffer());
    }
    self.execBroadCast = function () {
        self.ws.send('BROADCAST: ' + self.sendBuffer());
    }
    self.execPingRequest = function () {
        self.ws.send('SENDPING');
    }
    self.execCloseRequest = function () {
        self.ws.send('SENDCLOSE');
    }
    self.execCountRequest = function () {
        self.ws.send("SENDCOUNT");
    }
    self.execImageRequest = function () {
        self.ws.send("SENDIMAGE");
    }
    self.execLargeRequest = function () {
        var payLoad = "";
        for (var i = 0; i < 128000; i++)
        {
            payLoad += "z";
        }
        self.ws.send(payLoad);
    }
    self.execStartKeepAliveRequest = function () {
        self.ws.send("STARTKEEPALIVE");
    }
    self.execStopKeepAliveRequest = function () {
        self.ws.send("STOPKEEPALIVE");
    }

    self.send = function (m) {
        self.ws.send(m);
    }

    //init
    self.init = function(url) {
        if(!self.inited)
        {
            self.inited = true;
            self.url = url;
            self.ws = new DDSocketApp.SocketBase(url, self.socketCallback);       
            if(!self.ws.supported())
            {
                alert('sockets not supported in this browser!');
            }
        }
        ko.applyBindings(self);
    }

    //callback
    self.socketCallback = function(socketEvent, event)
    {
        //DDSocketApp.OpenEvent = "opened";
        //DDSocketApp.CloseEvent = "close";
        //DDSocketApp.MessageEvent = "message";
        //DDSocketApp.ErrorEvent = "error";

        var m = new DDTestApp.Msg(socketEvent, "event: '" + socketEvent + "' received. ");
        if (socketEvent == DDSocketApp.MessageEvent && event.data != null)
        {
            if (event.data != null && event.data.length > 1000)
            {
                m.append("Large message received: " + event.data.length);
                if(event.data.substr(0,10) == "data:image" )
                {
                    self.imageData(event.data);
                }
            }
            else
                m.append(event.data);
        }
        self.messages.push(m);

        switch(socketEvent)
        {
            case DDSocketApp.OpenEvent: 
                self.connected(true);
                break;
            case DDSocketApp.CloseEvent:
            case DDSocketApp.ErrorEvent:
                self.connected(false);
                break;            
        }
        
    }
    self.notEmpty = function (txt) {
        return txt != null && txt != "";
    }
}
