  MEMBER('cart.clw')

  include('CartClass.inc'),ONCE
  Include('JsonObject.inc'),ONCE

                              MAP
                              end!m

CartClass.Construct    PROCEDURE
  CODE
  SELF.Q &= NEW CartsQ
  SELF.ActiveCarts &= NEW OnlineQ
  
CartClass.Destruct    PROCEDURE
  CODE
  FREE(SELF.Q)
  DISPOSE(SELF.Q)
  FREE(SELF.ActiveCarts)
  DISPOSE(SELF.ActiveCarts)
  
CartClass.init                PROCEDURE(NetWebServer p_nws, WebSocketServerClass p_ws)
  CODE
  SELF.NWS &= p_nws
  SELF.WS &= p_ws
  
CartClass.Set                       PROCEDURE(STRING p_SessionID)
  CODE
  SELF.Pointer = 0
  SELF.SessionID = p_SessionID
  
CartClass.Next                PROCEDURE
RetVal                          BYTE
  CODE
  LOOP 
    SELF.Pointer += 1
    GET(SELF.Q, SELF.Pointer)
    if errorcode() 
      Retval = Level:Notify
      break
    end!if
    if SELF.Q.SessionID = SELF.SessionID
      BREAK
    end!if
  end!loop
  return RetVal
  
CartClass.AddLine                   procedure(STRING p_SessionID, STRING p_ProductID, LONG p_ItemCount, REAL p_Price)
  CODE
  SELF.Wait()
  CLEAR(SELF.Q)
  SELF.Q.ItemCount = p_ItemCount
  SELF.Q.Price = p_Price
  SELF.Q.Product = p_ProductID
  SELF.Q.SessionID = p_SessionID
  ADD(SELF.Q, SELF.Q.SessionID)
  
  SELF.Release()
  
CartClass.GetLineTotal        PROCEDURE()
  CODE
  return SELF.Q.ItemCount * SELF.Q.Price
  
CartClass.GetCartTotal              PROCEDURE(STRINg p_SessionID)
t                                     REAL
  CODE
  SELF.Wait()
  SELF.Set(p_SessionID)
  LOOP while not SELF.Next()
    t += SELF.GetLineTotal()
  end!loop
  SELF.Release()
  return t
  
CartClass.Wait                PROCEDURE
  CODE
  SELF.NWS._wait()

CartClass.Release             PROCEDURE
  CODE
  SELF.NWS._Release()
  
!--- implementation of websocket methods
  
CartClass.SetOnline           PROCEDURE(STRING p_SessionID, LONG p_SockID)
  CODE
  SELF.Wait()
  SELF.ActiveCarts.SessionID = p_SessionID
  GET(SELF.ActiveCarts, SELF.ActiveCarts.SessionID)
  if errorcode()
    SELF.ActiveCarts.SessionID = p_SessionID
    SELF.ActiveCarts.SockID = p_SockID
    SELF.ActiveCarts.OnlineDate = today()
    SELF.ActiveCarts.OnlineTime = clock()
    ADD(SELF.ActiveCarts,SELF.ActiveCarts.SessionID)
  ELSE
    SELF.ActiveCarts.SockID = p_SockID
    SELF.ActiveCarts.OnlineDate = today()
    SELF.ActiveCarts.OnlineTime = clock()
    PUT(SELF.ActiveCarts)
  end!if
  
  SELF.SendSalesNotification(p_SockID)
  SELF.Release()
  
CartClass.SetOffline                PROCEDURE(STRING p_SessionID)
  CODE
  SELF.ActiveCarts.SessionID = p_SessionID
  GET(SELF.ActiveCarts, SELF.ActiveCarts.SessionID)
  if ~errorcode()
    DELETE(SELF.ActiveCarts)
  end!if
  
CartClass.SendSalesNotification   procedure(LONG p_SockID)
jMessage                            JsonObject
SendBuffer                          StringTheory
nCount                              LONG
  CODE
  SELF.Wait()
  Access:SalesStaff.open()
  Access:SalesStaff.usefile()  
  SET(Sal:KeyID)
  LOOP while not access:SalesStaff.next()
    if SalesStaff.Online
      nCount += 1
    end!if
  end!loop
  jMessage.Add('MsgType','SalesNotification')
  jMessage.Add('StaffOnline', nCount,json:Numeric)
  jMessage.Stringify(SendBuffer)
  
  if p_SockID = -1
    SELF.WS.BroadcastText(SendBuffer)
  else
    SELF.WS.SendText(p_SockID, SendBuffer)
  end!if  
  Access:SalesStaff.close()
  SELF.Release()  
  
CartClass.HandleMessage       PROCEDURE(JSONObject p_o, LONG p_SockID)
jPayload                        JSONObject
jItems                          &JSONObject
jItem                           &JSONObject
Buffer                          StringTheory
  CODE
  case UPPER(p_o.GetPropertyValue('MsgType'))
  of 'GETSALESSTAFF'
    Access:SalesStaff.open()
    access:SalesStaff.UseFile()
    jPayload.Add('MsgType', 'StaffList')
    jItems &= jPayload.Add('Staff')
    SET(Sal:KeyName)
    LOOP while not Access:SalesStaff.next()
      jItem &= jItems.AddItem()
      jItem.Add('Name', Sal:Name)
      jItem.Add('ID', Sal:ID)
      jItem.Add('Online', Sal:Online, json:boolean)
    end!loop
    jPayload.Stringify(Buffer)
    
    Access:SalesStaff.close()
    SELF.WS.SendText(p_SockID, Buffer)
    
  OF 'SETSTATE'
    Access:SalesStaff.open()
    access:SalesStaff.UseFile()
    jPayload.Add('MsgType', 'SetStateResponse')
    SAL:ID = p_o.GetPropertyValue('ID')
    jPayload.Add('ID',Sal:ID)
    if ~access:SalesStaff.TryFetch(Sal:KeyID)
      Sal:Online = p_o.GetPropertyValue('State')
      if ~access:SalesStaff.TryUpdate()
        jPayload.Add('Result', 1,json:Boolean)
        jPayload.Add('Online', Sal:Online)
        !notify the clients
        MyCart.SendSalesNotification(-1)
      ELSE
        jPayload.Add('Result', 0,json:Boolean)
      end!if
    ELSE
      jPayload.Add('Result', 0,json:Boolean)
    end!if

    jPayload.Stringify(Buffer)    
    Access:SalesStaff.close()
    SELF.WS.SendText(p_SockID, Buffer)
    
  end!case        
  