var CartApp = CartApp || {};  // namespace for your app. Choose whatever your want, just don't use a . in the name

CartApp.VM = function () {  //defines a viewmodel 'class'. Note the absence of the new keyword. This is only a definition, not an instantiation
    var self = this;

    //property definitions
    self.inited = false;  //to prevent running initialization code more than once.
    self.ws = null;       //in the init, this property will be assigned an instance of the SocketBase class.
    
    self.handleMessage = function(data)
    {
        //here you can implement all messages from the server
        self.log('message received');
        switch(data.MsgType)
        {
            case "SalesNotification":
                if (data.StaffOnline > 0)
                {
                    $("#notification").show(800);
                }
                else
                {
                    $("#notification").hide(800);
                }
                break;
        }
    }

    //if you want to send something to the server, create a function for it here
    //example:   
    //msgobject: { code: 100, text: "This is an example object"}
    //mind you: a websocket will only allow you to call the server not other clients. 
    //the server however, does have the ability to send messages to specific clients or event broadcast messages
    //so, if you want to send a message to another client: tell the server to do so on your behalf.
    self.sendToServer = function (msgobject) {
        self.ws.send(JSON.stringify(msgobject));
    }

    //this method needs to be called from the page initialization code (jQuery ready)
    self.init = function (url) {
        if (!self.inited) {
            self.inited = true;
            self.ws = new DDSocketApp.SocketBase(url, self.handleSocketCallbacks);   //creates an instance of the socketbase class.
            if (!self.ws.supported())
                alert('Websockets are not supported!');
            else
                self.ws.connect();

            //event binding. You might want to add other event bindings here.
            $(document).unload(function () {    //just good housekeeping: if the page unloads, close the socket. (That will happen anyway)
                self.ws.close();
            });

            self.log('inited Cart app');
        }
    }


    //the rest is boilerplate code that you can change if needed
    self.handleSocketCallbacks = function(socketEventType, event )
    {
        if (socketEventType == DDSocketApp.MessageEvent && event.data != null) {
            self.log('received: ' + event.data);
            try {
                self.handleMessage(JSON.parse(event.data));
            }
            catch (e) {
                self.log('Error handling message! ' + e);
            }
        }

        switch (socketEventType) {
            case DDSocketApp.OpenEvent:
                self.log('opened')
                break;
            case DDSocketApp.CloseEvent:
                self.log('closed');
            case DDSocketApp.ErrorEvent:
                self.log('error')
                break;
        }
    }

    self.log = function(msg)
    {
        if(console && console.log )
        {
            console.log(msg);
        }
    }
}