  MEMBER

  include('TaxiMapClass.inc'),ONCE
  INCLUDE('CWSYNCHM.INC'),ONCE
  INCLUDE('JSONObject.inc'),ONCE

                              MAP
                              end!m

TaxiMapClass.Construct    PROCEDURE
  CODE
  SELF.SyncRoot &= NewCriticalSection()
  SELF.Q &= NEW TaxiQ
  SELF.Rides &= NEW RidesQ
  
TaxiMapClass.Destruct    PROCEDURE
  CODE
  DISPOSE(SELF.SyncRoot)
  FREE(SELF.Q)
  DISPOSE(SELF.Q)
  FREE(SELF.Rides)
  DISPOSE(SELF.Rides)
  
TaxiMapClass.SetWebSocketServer PROCEDURE(WebSocketServerClass p_ws)
  CODE
  SELF.WS &= p_ws
  
TaxiMapClass.HandleOpen       PROCEDURE(LONG p_SockID)
CP                              CriticalProcedure
jMessage                        JSONObject
SendBuffer                      StringTheory
TaxiAdded                       BYTE
  CODE
  CP.init(SELF.SyncRoot)
  
  !logged in? If so, create the taxi
  if SELF.WS.GetSessionLoggedIn(p_SockID)
    SELF._addTaxiFromSession(p_SockID)
    TaxiAdded = true
  end!if
  
  jMessage.Add('MsgType', 'Open')
  !send list of all taxis to all
  SELF._AppendTaxiList(jMessage)
  if TaxiAdded
    SELF._AppendRideList(jMessage)
  end!if
  
  jMessage.Stringify(SendBuffer)
  SELF.WS.BroadcastText(SendBuffer)
  
TaxiMapClass.HandleClose                 PROCEDURE(LONG p_SockID)
CP CriticalProcedure
  CODE
  CP.init(SELF.SyncRoot)
  !update q
  SELF.Q.SockID = p_SockID
  GET(SELF.Q, SELF.Q.SockID)
  if ~errorcode()
    DELETE(SELF.Q)
  end!if
  !send list of all taxis to all 
  SELF._sendTaxiList(p_SockID, true)

TaxiMapClass.HandleMessage               procedure(LONG p_SockID, JSONObject p_o)
CP                                          CriticalProcedure
jMessage                                    JSONObject
SendBuffer                                  StringTheory
personCount                                 LONG
  CODE
  CP.init(SELF.SyncRoot)
  case UPPER(p_o.GetPropertyValue('MsgType'))
  of 'POSITIONUPDATE'
    SELF.Q.SockID = p_SockID
    GET(SELF.Q,SELF.Q.SockID)
    if ~errorcode()
      SELF.Q.X = p_o.GetPropertyValue('X')
      SELF.Q.Y = p_o.GetPropertyValue('Y')
      PUT(SELF.Q)

      SendBuffer.SetValue('')
      jMessage.Add('MsgType','PositionUpdate')
      jMessage.Add('SockID', p_SockID, json:Numeric)
      jMessage.Add('X',SELF.Q.X, json:Numeric)
      jMessage.Add('Y',SELF.Q.Y, json:Numeric)
      jMessage.Stringify(SendBuffer)
      SELF.WS.BroadcastText(SendBuffer)
    end!if
  of 'NEWRIDE'
    !no broadcast, instead, only signal large enough taxis
    personCount = SELF.WS.GSV(p_SockID, 'PersonCountNewRide')
    SELF._addRide(p_SockID, personCount, p_o)

    SendBuffer.SetValue('')
    jMessage.Add('MsgType','NewRideID')
    jMessage.Add('SockID', p_SockID, json:Numeric)
    jMessage.Add('ID', SELF.Rides.ID, json:Numeric)
    jMessage.Stringify(SendBuffer)
    SELF.WS.SendText(p_SockID, SendBuffer)
    
  of 'PICKUP'
    !remove ride 
    SELF._removeRide(p_SockID, p_o)
      
  of 'PLACEEVENT'
  end!case
  
!-------------------------------------------------------
!Procedures below - prefixed with _ - should not be called directly
!without synchronisation code.
!-----------------------------------------------------
TaxiMapClass._addRide         PROCEDURE(LONG p_SockID, LONG p_PersonCount, JSONObject p_o)
t                               LONG
jMessage                        JSONObject
SendBuffer                      StringTheory
  CODE
  CLEAR(SELF.Rides)
  SELF.LastRideID += 1
  SELF.Rides.ID = SELF.LastRideID
  SELF.Rides.PersonCount = p_PersonCount
  SELF.Rides.X = p_o.GetPropertyValue('X')
  SELF.Rides.Y = p_o.GetPropertyValue('Y')
  ADD(SELF.Rides, SELF.Rides.ID)
  
  jMessage.Add('MsgType','NewRide')
  SELF._RideToObject(jMessage)
  
  jMessage.Stringify(SendBuffer)
  
  LOOP t = 1 to records(SELF.Q)
    GET(SELF.Q, t)
    if SELF.Q.Seats >= p_PersonCount AND SELF.Q.SockID <> p_SockID
      SELF.WS.SendText(SELF.Q.SockID, SendBuffer)
    end!if
  end!loop
  
TaxiMapClass._removeRide      procedure(LONG p_sockID, JSONObject p_o)
jMessage                        JSONObject
SendBuffer                      StringTheory
  CODE
  SELF.Rides.ID = p_o.GetPropertyValue('Id')
  GET(SELF.Rides, SELF.Rides.ID)
  if ~errorcode()
    jMessage.Add('MsgType','PickedUp')
    jMessage.Add('ID', SELF.Rides.ID, json:Numeric)
    jMessage.Stringify(SendBuffer)
    
    SELF.WS.BroadcastText(SendBuffer) !send it to all
    DELETE(SELF.Rides)
  end!if
  
TaxiMapClass._sendTaxiList    PROCEDURE(LONG p_SockID, BYTE p_BroadCast = false)
t                                           LONG
jMessage                                    JSONObject
jItems                                      &JSONObject
jItem                                       &JSONObject
SendBuffer                                  StringTheory
  CODE
  jMessage.Add('MsgType','Open')
  SELF._AppendTaxiList(jMessage)      
  
  jMessage.Stringify(SendBuffer)

  if p_BroadCast
    SELF.WS.BroadcastText(SendBuffer)
  else
    SELF.WS.SendText(p_SockID,SendBuffer)
  end!if

TaxiMapClass._AppendTaxiList  procedure(JSONObject p_o)
jItems                          &JSONObject
jItem                           &JSONObject
t                               LONG
  CODE
  jItems &= p_o.Add('Taxis')
  
  LOOP t = 1 to records(SELF.Q)
    GET(SELF.Q,t)
    jItem &= jItems.AddItem()
    SELF._TaxiToObject(jItem)
  end!loop
 
TaxiMapClass._AppendRideList    PROCEDURE(JSONObject p_o)
r                                 LONG
jItems                            &JSONObject
jItem                             &JSONObject
  CODE
  jItems &= p_o.Add('Rides')
  
  LOOP r = 1 to records(SELF.Rides)
    GET(SELF.Rides,r)
    if SELF.Rides.PersonCount <= SELF.Q.Seats !only show rides that the taxi can hold
      jItem &= jItems.AddItem()
      SELF._RideToObject(jItem)
    end!if
  end!loop
      
TaxiMapClass._addTaxiFromSession  PROCEDURE(LONG p_SockID)
SendBuffer                      StringTheory
jMessage                        JSONObject
  CODE
  CLEAR(SELF.Q)
  SELF.Q.DriverName = SELF.WS.GSV(p_SockID, 'DriverName')
  SELF.Q.Seats = SELF.WS.GSV(p_SockID, 'Seats')
  SELF.Q.SockID = p_SockID
  ADD(SELF.Q, SELF.Q.SockID)

  jMessage.Add('MsgType', 'Open')
  SELF._TaxiToObject(jMessage)
  jMessage.Add('Me', true, json:Boolean)
  jMessage.Stringify(SendBuffer)
  
  SELF.WS.SendText(p_SockID, SendBuffer)
  
TaxiMapClass._TaxiToObject    procedure(JSONObject p_o)
  CODE
  p_o.Add('DriverName',CLIP(SELF.Q.DriverName))
  p_o.Add('SockID',SELF.Q.SockID, json:Numeric)
  p_o.Add('Seats', SELF.Q.Seats, json:Numeric)
  p_o.Add('X', SELF.Q.X, json:Numeric)
  p_o.Add('Y', SELF.Q.Y, json:Numeric)
  
TaxiMapClass._RideToObject    PROCEDURE(JSONObject p_o)
  CODE
  p_o.Add('ID',SELF.Rides.ID, json:Numeric)
  p_o.Add('X',SELF.Rides.X, json:Numeric )
  p_o.Add('Y',SELF.Rides.Y, json:Numeric )
  p_o.Add('Personcount', SELF.Rides.PersonCount, json:Numeric)
  