var DDTaxiApp = DDTaxiApp || {};

DDTaxiApp.Taxi = function (data) {
    var self = this;

    self.sockId = 0;
    self.driver = "";
    self.seats = 0;
    self.x = 0;
    self.y = 0;

    if (data) {
        self.sockId = data.SockID;
        self.driver = data.DriverName;
        self.seats = data.Seats;
        self.x = data.X;
        self.y = data.Y;
    }
};

DDTaxiApp.VM = function()
{
    var self = this;

    self.taxis = new Array();
    self.taxi = null;
    self.ws = null;
    self.inited = false;
    self.$taxiImg = $("#me");
    self.taxiImg = self.$taxiImg.get(0);
    self.taxiVisible = false;
    self.$others = $("#others");
    self.othersCtx = null;
    self.otherImg = document.getElementById('otherImg');
    self.imgw = 30;
    self.imgh = 21;
    self.newRideContainer = null;
    self.$newRideContainer = null;
    self.map = document.getElementById('map');
    self.$lastRide = null;

    self.handleMessage = function(msg)
    {
        switch(msg.MsgType)
        {
            case "Open":
                self.handleOpenMessage(msg);
                break;
            case "PositionUpdate":
                self.handlePositionUpdate(msg);
                break;
            case "NewRide":
                self.handleNewRide(msg);
                break;
            case "NewRideID":
                self.handleNewRideId(msg);
                break;
            case "PickedUp":
                self.handlePickedUp(msg);
                break;
            //case ""
        }
    }

    self.handleOpenMessage = function(msg)
    {
        if (msg.Taxis != null) {
            self.taxis = new Array();
            $.each(msg.Taxis, function (index, val) {
                var taxi = new DDTaxiApp.Taxi(val);
                if (self.taxi == null || self.taxi.sockId != taxi.sockId)
                    self.taxis.push(taxi);
            });
        }
        if(msg.Rides != null )
        {
            $.each(msg.Rides, function (index, val) {
                self.addRide(val.ID, val.X, val.Y);
            });
        }
        if(msg.DriverName != null)
            self.taxi = new DDTaxiApp.Taxi(msg);

        self.draw();
    }

    self.handlePositionUpdate = function(msg)
    {
        $.each(self.taxis, function(index,val){
            if(val.sockId == msg.SockID)
            {
                val.x = msg.X;
                val.y = msg.Y;
                self.draw();
                return false;
            }
        });
    }

    self.handleDrop = function (ev) {
        ev.preventDefault();
        if (ev.originalEvent.dataTransfer.getData('text') == "taxi")
        {
            self.taxi.x = ev.originalEvent.offsetX;
            self.taxi.y = ev.originalEvent.offsetY;
            self.updateTaxiPosition();
        }
        else { //ride
            var $i = self.addRide(-1,ev.originalEvent.offsetX,ev.originalEvent.offsetY );

            //store newly added ride. It still needs to get a server id (to be able to remove it when a taxi picks up 
            self.$lastRide = $i;
            //hide the new ride image below the dispatch form
            self.$newRideContainer.addClass('hide');

            //notify server of new ride
            var toSend = { msgType: 'NewRide', x: ev.originalEvent.offsetX, y: ev.originalEvent.offsetY };
            self.ws.send(JSON.stringify(toSend));
        }
        self.draw();
    };

    self.addRide = function(id,x,y)
    {
        var $i = $('<img src="images/newride.png" class="ride"/>');
        $i.appendTo(self.map)
            .css({ left: x, top: y })
            .data('id', id)
            .on('dragstart', function (ev) { ev.preventDefault(); });

        return $i;
    }

    self.handlePickup = function (ev) {
        ev.preventDefault();
        if (ev.originalEvent.dataTransfer.getData('text') == "taxi") {
            //pickup
            var $ride = $(ev.target);
            var id = $ride.data('id');
            var pos = $ride.position();
            var toSend = { msgType: 'Pickup', id: id };
            self.ws.send(JSON.stringify(toSend));
            
            self.taxi.x = pos.left;
            self.taxi.y = pos.top;
            self.updateTaxiPosition();
            self.draw();
        }
    }

    self.handlePickedUp = function (msg) {
        var id = msg.ID;
        $(".ride").each(function (index, el) {
            var $el = $(el);
            var rideId = $el.data('id');
            if(rideId == id)
            {
                $el.remove();
                return false; //stop the each
            }
        });
    }

    self.handleNewRide = function(msg)
    {
        var $i = $('<img src="images/newride.png" class="ride"/>');
        $i.appendTo(self.map);
        $i.css({ left: msg.X, top: msg.Y }).data('id', msg.ID);
    }

    self.handleNewRideId = function(msg)
    {
        if(self.$lastRide != null)
        {
            self.$lastRide.data('id', msg.ID);
        }
    }

    //drawing
    self.draw = function()
    {
        if(self.taxi != null)
        {
            if (!self.taxiVisible)
            {
                self.taxiVisible = true;
                self.$taxiImg.show();
            }
            self.$taxiImg.css({ left: self.taxi.x, top: self.taxi.y });
        }
        self.othersCtx.clearRect(0, 0, 800, 800);
        $.each(self.taxis, function (index, val) {
            self.othersCtx.drawImage(self.otherImg, val.x, val.y, self.imgw, self.imgh);
            self.othersCtx.strokeText(val.driver, val.x, val.y+30);
        });
    }

    self.updateTaxiPosition = function () {
        var toSend = {
            msgType : "positionUpdate"
            , x : self.taxi.x
            , y: self.taxi.y
        };
        self.ws.send(JSON.stringify(toSend));
    }

    //websockets callback method
    self.handleCallback = function (socketEvent, event)
    {
        if (socketEvent == DDSocketApp.MessageEvent && event.data != null) {
            self.log('received: ' + event.data);
            try{
                self.handleMessage(JSON.parse(event.data));
            }
            catch(e)
            {
                self.log('Error handling message! ' + e);
            }
        }

        switch (socketEvent) {
            case DDSocketApp.OpenEvent:
                self.log('opened')                
                break;
            case DDSocketApp.CloseEvent:
            case DDSocketApp.ErrorEvent:
                self.log('closed');
                break;
        }
    }

    //general
    self.init = function (url, mapid) {
        if(!self.inited)
        {
            self.inited = true;
            self.ws = new DDSocketApp.SocketBase(url, self.handleCallback);
            if (!self.ws.supported())
                alert('Websockets are not supported!');
            else
                self.ws.connect();

            //setup canvas
            self.othersCtx = self.$others.get(0).getContext("2d");
            self.$others.get(0).width = 800;
            self.$others.get(0).height = 800;
            self.othersCtx.font = "8px Verdana";
            self.othersCtx.strokeStyle = "#000";

            self.newRideContainer = document.getElementById('NewRideContainer');
            self.$newRideContainer = $(self.newRideContainer);

            //bind events
            self.$taxiImg.bind('dragstart', function (ev) {
                ev.originalEvent.dataTransfer.setData('text', 'taxi');
            });

            self.$others.bind('dragover', function (ev) {
                ev.preventDefault();
            }).bind('drop', self.handleDrop);
            
            $(document).on('dragover', ".ride", function (ev) { //attach it to all rides 
                ev.preventDefault();
            }).on('drop', ".ride", self.handlePickup);

            //events from other code
            $(document).bind('NewRide', function () {
                self.$newRideContainer.removeClass("hide");
            });
        }
    }

    self.log = function (msg) {
        if(console && console.log)
        {
            console.log(msg);
        }
    }
}